#include <stdio.h>
#include <time.h>
#include <stdarg.h>
#include <sys/time.h>
#include <assert.h>
#include <stdlib.h>

#ifdef __K1__
#include <HAL/hal/hal.h>
#endif

void userPrintf(const char* format, ...) {
#if !defined(__sparc__) && !defined(__K1__)
    va_list(args);
    va_start(args, format);
    printf("USER_PROCESS> ");
    vprintf(format, args);
    va_end(args);
#endif    
}

void my_micro_sleep(long arg_micro) {
  struct timespec timeOut,remains; 
  timeOut.tv_sec = 0; 
  timeOut.tv_nsec = (1000L)*arg_micro; 
  nanosleep(&timeOut, &remains); 
}

  
void SPLIT__BODY() {     
    my_micro_sleep( 400); // +600 write
}

void PROCESS_A_BODY() {
  static int k = 0;
  if (k%2==0) {
    my_micro_sleep(5000); // +1000 read = 6000
  } else {
    // immitate exceptional resource demand
    my_micro_sleep(11000); // +1000 read = 12000 
  }
  k++;
}

void  PROCESS_B_BODY() { 
    my_micro_sleep(5000); 
}


#ifdef __sparc__
// Gantt measurements
int gantt_procnum()
{
    unsigned int tmp;

    // IDENT is stored in ASR17
    asm volatile (
            "rd %%asr17, %0 \n\t"
            "srl %0, 28, %0 \n\t"
            : "=r"(tmp)
            :
            : "%0"
            );
    return tmp;
}
#elif defined(__K1__)
int gantt_procnum() {
  return  __k1_get_cpu_id();
}
#endif


#if !defined(__sparc__) && !defined(__K1__)
void mark_start(int id){}
void mark_end(){}
#else


int get_cpu_id() {
    return gantt_procnum();
}

#define P_NUM 10
#define ENTRY_NUM 256
#define STOP_NUM 50
typedef struct chart{
    int start[P_NUM][ENTRY_NUM];
    int finish[P_NUM][ENTRY_NUM];
    int id[P_NUM][ENTRY_NUM];
    int curr[P_NUM];
    int overhead;
}Chart;
Chart gantt;
int interNum[ENTRY_NUM];


void chart_init(){
    int i,j;
    for(i=0;i<P_NUM;i++){
        gantt.curr[i] = 0;
    }
    for (j=0; j<ENTRY_NUM; j++) {
        interNum[j] =0;
    }
}

void mark_start(int id){
    struct timeval tv;
    int cpu = get_cpu_id(); 
    //cluster: " <<  __k1_get_cluster_id() << "; core: " << __k1_get_cpu_id();
    //printf("WRITE at CPU=%d ID=%d CURR=%d\n", cpu, id, gantt.curr[cpu]);
    //PP debug
    //if (id <20) {
    //    printf("> start %d\n", id);
    //}
    assert( cpu < P_NUM);
    assert( gantt.curr[cpu]<ENTRY_NUM);
    gettimeofday(&tv,NULL);
    int time = 1000000*(int)tv.tv_sec+(int)tv.tv_usec;
    gantt.start[cpu][gantt.curr[cpu]] = time-gantt.overhead;
    gantt.id[cpu][gantt.curr[cpu]] = id;
    //printf("process %d starting at %d on core %d\n",id,time,cpu);
}

void mark_end(){
    struct timeval tv;
    int cpu = get_cpu_id(); 
    gettimeofday(&tv,NULL);
    int time = 1000000*(int)tv.tv_sec+(int)tv.tv_usec;
    gantt.finish[cpu][gantt.curr[cpu]++] = time-gantt.overhead;
    //printf("process finished on cpu %d at time %d\n",cpu,time);
}

void print_gantt(int niter){
    int cpu = get_cpu_id(); 
    assert( cpu < P_NUM);
    if(gantt.curr[cpu]>=STOP_NUM-1){
        //for(int j=0;j<5000;j++)printf("preparing to print...\n");
        //	printf("about to print...\n");
        for(int c=0;c<P_NUM;c++){
            //         printf("CPU %d\n",c);
            int i=0;
            while(gantt.start[c][i]!=0||i==0){
                printf("%d\t%d\t%d\tP%d - ET=%d",c,gantt.start[c][i],gantt.finish[c][i],gantt.id[c][i],gantt.finish[c][i]-gantt.start[c][i]);
                if(gantt.id[c][i]==20) printf(" NumItnter=%d",interNum[i]);
                printf("\n");
                i++;
            }
        }
        fflush(stdout);
        exit(0);
    }else{
        //	printf("it=%d\n",gantt.curr[1]);
    }
}

void reset_gantt_time(){
    struct timeval tv;
    gettimeofday(&tv,NULL);
    int time = 1000000*(int)tv.tv_sec+(int)tv.tv_usec;
    gantt.overhead = time;
}



void count_inter(){
    int cpu = get_cpu_id();
    assert( gantt.curr[cpu]<ENTRY_NUM);
    interNum[gantt.curr[cpu]]++;
}

#endif
