#include "global.h"

#include <pthread.h>
#ifdef __K1__
#include <HAL/hal/hal.h>
#endif 

#include <cstring>
#include <string>
#include <sstream>
#include <cassert>

using namespace std;

//======================================
// Application-specific!
#define P_NUM 3
#define ENTRY_NUM 128
#define STOP_NUM 20
//======================================

#ifdef __K1__
#include <HAL/hal/hal.h>
#else 
int __k1_get_cpu_id() {
    return 0;
}
#endif

typedef struct chart{
    int start[P_NUM][ENTRY_NUM];
    int finish[P_NUM][ENTRY_NUM];
    int id[P_NUM][ENTRY_NUM];
    int curr[P_NUM];
    int overhead;
    int interNum[ENTRY_NUM];
}Chart;
Chart gantt;


void chart_init(){
    for(int i=0;i<P_NUM;i++){
        gantt.curr[i] = 0;
    }
}

void mark_start(int id){
    struct timeval tv;
    int cpu = __k1_get_cpu_id(); 
    //cluster: " <<  __k1_get_cluster_id() << "; core: " << __k1_get_cpu_id();
    //printf("WRITE at CPU=%d ID=%d CURR=%d\n", cpu, id, gantt.curr[cpu]);
    assert( cpu < P_NUM);
    assert( gantt.curr[cpu]<ENTRY_NUM);
    gettimeofday(&tv,NULL);
    int time = 1000000*(int)tv.tv_sec+(int)tv.tv_usec;
    gantt.start[cpu][gantt.curr[cpu]] = time-gantt.overhead;
    gantt.id[cpu][gantt.curr[cpu]] = id;
	gantt.finish[cpu][gantt.curr[cpu]] = 0;
    if (id==20) {
	    gantt.interNum[gantt.curr[cpu]] = 0;
    }
    //printf("process %d starting at %d on core %d\n",id,time,cpu);
}

void mark_end(){
    struct timeval tv;
    int cpu = __k1_get_cpu_id(); 
    gettimeofday(&tv,NULL);
    int time = 1000000*(int)tv.tv_sec+(int)tv.tv_usec;
    gantt.finish[cpu][gantt.curr[cpu]++] = time-gantt.overhead;
    //printf("process finished on cpu %d at time %d\n",cpu,time);
}

void print_gantt(int niter){
    int cpu = __k1_get_cpu_id(); 
    assert( cpu < P_NUM);
    if(gantt.curr[cpu]>=STOP_NUM-1){
        //for(int j=0;j<5000;j++)printf("preparing to print...\n");
        //  printf("about to print...\n");
        for(int c=0;c<P_NUM;c++){
            //         printf("CPU %d\n",c);
            int i=0;
            while(gantt.start[c][i]!=0||i==0){
                printf("%d\t%d\t%d\tP%d - ET=%d",c,gantt.start[c][i],gantt.finish[c][i],gantt.id[c][i],gantt.finish[c][i]-gantt.start[c][i]);
                if(gantt.id[c][i]==20) printf(" NumItnter=%d",gantt.interNum[i]);
                printf("\n");
                i++;
            }
        }
        while(true);
    }else{
        //  printf("it=%d\n",gantt.curr[1]);
    }
}

void reset_gantt_time(){
    struct timeval tv;
    gettimeofday(&tv,NULL);
    int time = 1000000*(int)tv.tv_sec+(int)tv.tv_usec;
    gantt.overhead = time;
}

void count_inter(){
	int cpu = __k1_get_cpu_id();
	assert( gantt.curr[cpu]<ENTRY_NUM);
	gantt.interNum[gantt.curr[cpu]]++;
}


//================================================================
//
//  Not related to Gantt charts but useful: busy-execute for given 
//  number of milliseconds
//
//================================================================
static int loop_a = 0;

void execute_loop_for_msec(long duration) {
//  Does not work on Kalray MPPA:
//    clock_t endcl = ::clock() + (duration*CLOCKS_PER_SEC)/1000;
//    while ( ::clock() < endcl) {
//        int k;
//        for (k=0; k<1000; k++) { loop_a = k + 1;}            
//    }            

   struct timeval starttim;
   gettimeofday(&starttim, NULL);
 
   while (true) {
     int k;
     for (k=0; k<1000; k++) { loop_a = k + 1;}
     struct timeval endtim;
     gettimeofday(&endtim, NULL);
     long diff_sec  = endtim.tv_sec  - starttim.tv_sec;
     long diff_usec = endtim.tv_usec - starttim.tv_usec;
 
     long diff_msec = diff_sec*1000 + diff_usec/1000;
     if (diff_msec>=duration) break;
   }
}

const char* thread_info(char* buf, int sz) {
    stringstream infostr;
#ifdef __K1__
    infostr << "thread: " << pthread_self() << "; cluster: " <<  __k1_get_cluster_id() << "; core: " << __k1_get_cpu_id(); 
#else
    infostr << "thread: " << pthread_self();
#endif
    strncpy(buf, infostr.str().c_str(),sz);
    return buf;
}

