//package graphoutput;
import java.io.*;
import java.util.*;


public class GanttChart 
{
	// TODO: This is not yet working. I have to fix the height of the rectangle properly.
	private double rectangleHeight = 0.7;

	private HashSet<String> tasks;
	private HashMap<String, Double> resources;
	private HashMap<String, String> colorBook;
	List<Record> records;

	public GanttChart()
	{
		records = new ArrayList<Record>();
		tasks = new HashSet<String>();
		resources = new HashMap<String, Double>();
		colorBook = new HashMap<String, String>();
	}	

	private class Record implements Comparable<Record>
	{
		String processor;
		long startTime;
		long endTime;
		String actorName;		

		double bottomLeftX;
		double bottomLeftY;
		double topRightX;
		double topRightY;
		double yPosition;

		String color;

		public int compareTo(Record anotherInstance)
		{
			String tempStr1 = actorName.replaceAll("[0-9]+","");
			String tempStr2 = anotherInstance.actorName.replaceAll("[0-9]+","");

			if(tempStr1.equals(tempStr2))
			{
				String tStr1 = actorName.replaceAll("[^\\d.]", "");
				String tStr2 = anotherInstance.actorName.replaceAll("[^\\d.]", "");
				return Integer.parseInt(tStr1) - Integer.parseInt(tStr2); 				
			}
			else	
				return actorName.compareTo(anotherInstance.actorName);
		}

		Record(String processor, int startTime, int endTime, String actorName)
		{
			this.processor = processor;
			this.startTime = startTime;
			this.endTime = endTime;
			this.actorName = actorName;
		}
		
		Record(String recordLine)
		{
			recordLine = recordLine.trim().replaceAll("\t", " ");
			recordLine = recordLine.trim().replaceAll("( )+", " ");			
			String[] tokens = recordLine.split(" ");			
			processor = tokens[0];
			startTime = Long.parseLong(tokens[1]);
			endTime = Long.parseLong(tokens[2]);
			actorName = tokens[3];
		}
	}

	private void calculateRectangles() 
	{		
		for(int i=0;i<records.size();i++)
		{
			records.get(i).yPosition = resources.get(records.get(i).processor);
			records.get(i).bottomLeftX = records.get(i).startTime;
			records.get(i).bottomLeftY = records.get(i).yPosition - 0.5 * rectangleHeight;
			records.get(i).topRightX = records.get(i).endTime;
			records.get(i).topRightY = records.get(i).yPosition + 0.5 * rectangleHeight;
			records.get(i).color = colorBook.get(records.get(i).actorName);
		}		
	}

	private void loadColorBook(String colorFileName)
	{
		try
		{
			// Open the file that is the first 
			// command line parameter
			FileInputStream fStream = new FileInputStream(colorFileName);
			// Get the object of DataInputStream
			DataInputStream in = new DataInputStream(fStream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String strLine;

			int first = 0;
			//Read File Line By Line
			while ((strLine = br.readLine()) != null)
			{
				if(first == 0)
				{
					first++;
					if(strLine.equals("[Colors]") == false)
					{
						br.close();
						in.close();
						fStream.close();
						throw new RuntimeException("Colors header missing.");
					}
				}
				else
				{
					strLine = strLine.replace(":", "");
					String split[] = strLine.split(" ");
					colorBook.put(split[0], split[1]);
				}				
			}
			
			br.close();
			in.close();
			fStream.close();
			
		}catch (Exception e) {}		
	}

	private void generateUniqueTasksResources ()
	{
		tasks.clear();
		resources.clear();

		SortedSet<String> uniqueProc = new TreeSet<String>();

		for(int i=0;i<records.size();i++)
			uniqueProc.add(records.get(i).processor);

		int count=0;
		Iterator<String> procIter = uniqueProc.iterator();
		while(procIter.hasNext())
		{
			String proc = procIter.next();			
			resources.put(proc, (double) (uniqueProc.size() - count));
			count++;
		}

		for(int i=0;i<records.size();i++)		
			tasks.add(records.get(i).actorName);		
	}

	double calculateXMax()
	{
		double max = Double.MIN_VALUE;

		for(int i=0;i<records.size();i++)
			if(records.get(i).endTime > max)
				max = records.get(i).topRightX;

		return max;		
	}

	private void generatePdf(String outputFileName)
	{
		String pdfGenerationConfigFile = "inputFiles/gantt_chart/plotPdf.gpl";
		// System.out.println("We assume that you have Gnuplot installed on your system !!");
		String configFile = outputFileName.replace("pdf", "gpl");

		try 
		{
			Runtime r = Runtime.getRuntime();
			String command = "gnuplot " + configFile + " " + pdfGenerationConfigFile;
			Process p = r.exec(command);
			p.waitFor();

			// If there is any error, it will get printed here.
			BufferedReader b = new BufferedReader(new InputStreamReader(p.getInputStream()));
			String line = "";

			while ((line = b.readLine()) != null) {
				System.out.println(line);
			}

			// Generated the file, now lets move it to appropriate location.
			command = "mv -f output.pdf " + outputFileName;
			p = r.exec(command);
			p.waitFor();

		} 
		catch (IOException e) { e.printStackTrace(); } 
		catch (InterruptedException e) { e.printStackTrace(); }

		System.out.println("Finished generating PDF.");
	}
/*
	public void plotChart (Map<String,Integer> model, Map<String,Integer> taskDuration, 
								String outputFileName, String colorConfigFile, boolean generatePdfOutput)
	{
		// Build The Records
		buildRecordsFromModel(model, taskDuration);
		// Load the color book
		loadColorBook(colorConfigFile);
		// Generate Unique Tasks and Resources
		generateUniqueTasksResources();
		// Calculate the Rectangles in the plot.
		calculateRectangles();
		// Generate the GnuPlot File
		genereteGnuConfigFile(outputFileName);
		// Generate the PDF Output.
		if(generatePdfOutput)
			generatePdf(outputFileName);
	}*/

	public void plotChart (String timingFileName, String outputFileName, String colorConfigFile, boolean generatePdfOutput)
	{
		// Build The Records
		buildRecordsFromFile(timingFileName);
		// Load the color book
		loadColorBook(colorConfigFile);
		// Generate Unique Tasks and Resources
		generateUniqueTasksResources();
		// Calculate the Rectangles in the plot.
		calculateRectangles();
		// Generate the GnuPlot File
		genereteGnuConfigFile(outputFileName);
		// Generate the PDF Output.
		if(generatePdfOutput)
			generatePdf(outputFileName);

	}
	/*
	private void buildRecordsFromModel(Map<String, Integer> model, Map<String,Integer> taskDuration) 
	{
		records.clear();
		
		Iterator<String> modelIter = model.keySet().iterator();
		while(modelIter.hasNext())
		{
			String key = modelIter.next();
			if(key.startsWith("x"))
			{
				String actorName = key.substring(1);
				int startTime = model.get(key);
				String processor = model.get("cpu"+actorName);
				int endTime = startTime;
				String split[] = actorName.split("_");
				int duration = taskDuration.get(split[0]);
				endTime += duration;
				
				records.add(new Record(processor, startTime, endTime, actorName));				
			}
		}
	}*/

	private void buildRecordsFromFile(String fileName)
	{
		try
		{
			// Open the file that is the first 
			// command line parameter
			FileInputStream fInStream = new FileInputStream(fileName);
			// Get the object of DataInputStream
			DataInputStream in = new DataInputStream(fInStream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String strLine;
			
			records.clear();

			//Read File Line By Line
			while ((strLine = br.readLine()) != null)	
				records.add(new Record(strLine));
			
			br.close();
			in.close();
			fInStream.close();

		}catch (Exception e){ System.err.println("Error: " + e.getMessage());}
	}

	private void genereteGnuConfigFile(String fileName)
	{
		try
		{
			String configFile = fileName.replace("pdf", "gpl");
			
			// Create file 
			FileWriter fstream = new FileWriter(configFile);
			BufferedWriter out = new BufferedWriter(fstream);			

			double xmin = 0;
			double xmax = calculateXMax();
			double ymin = 0 + (rectangleHeight / 2);
			double ymax = resources.size() + 1 - (rectangleHeight / 2);

			// Generate the outputFile.
			out.write("set xrange ["+String.format("%.6f",xmin)+":"+String.format("%.6f",xmax)+"]\n");
			out.write("set yrange ["+String.format("%.6f",ymin)+":"+String.format("%.6f",ymax)+"]\n");
			out.write("set autoscale x\n");
			out.write("set xlabel \"time\"\n");
			out.write("set ylabel \"Processors\"\n");
			out.write("set title \"Gantt chart\"\n");
			out.write("set ytics (");

			int i=0;
			Iterator<String> procIter = resources.keySet().iterator();
			while(procIter.hasNext())
			{
				String proc = procIter.next();
				out.write("\"" + proc + "\" "+ String.format("%.0f",resources.get(proc)));
				if(i<resources.size()-1)
					out.write(", ");
				i++;
			}

			out.write(")\n");
			// out.write("set ytics ("1" 5, "0" 6, "3" 3, "2" 4, "5" 1, "4" 2)");
			out.write("set key outside width +2\n");
			out.write("set grid xtics\n");
			out.write("set palette model RGB\n");
			out.write("unset colorbox\n");

			for(i=0;i<records.size(); i++)
			{
				out.write("set object "+ (i+1) +" rectangle from " 
						+ String.format("%.6f",records.get(i).bottomLeftX) 
						+", " + records.get(i).bottomLeftY 
						+ " to " + String.format("%.6f",records.get(i).topRightX) + ", " 
						+ records.get(i).topRightY 
						+ " fillcolor rgb "
						+ records.get(i).color + " fillstyle solid 0.8\n");
				if(records.get(i).topRightX - records.get(i).bottomLeftX > 0)
					out.write("set label "+ (i+1) +" at "
							+ String.format("%.1f", ((double)(records.get(i).startTime + records.get(i).endTime)/2))
							+ " , " + String.format("%.0f", (records.get(i).bottomLeftY + records.get(i).topRightY)/2) 
							+ " rotate by 90 left" + "  \""
							+ records.get(i).actorName +"\" front center \n");
			}

			Collections.sort(records);

			out.write("plot ");
			
			//
			//	this part of the code has been modified in order to print only one instance of task with the same name
			//
			String prev = new String("$$$$");
			String next = null;
			for(i=0;i<records.size();i++)
			{
				
				next = records.get(i).actorName;
				if(next.compareTo(prev)!=0){
					if(i != 0)
						out.write(", \\\n\t");
					out.write("-1 title \"" + records.get(i).actorName + "\" with lines linecolor rgb " + 
							records.get(i).color + "  linewidth 6");
					
				}
				prev = next;
			}

			//Close the output stream
			out.close();
		}catch (Exception e){ System.err.println("Error: " + e.getMessage());}
	}
}
