#ifndef COMPONENT_H
#define COMPONENT_H

#include <cstdio>

class Compound;
class Scheduler;
class LogicalClock;
class Clock;
/***********************************
 * Class: Component
 * Description: component base class
 ***********************************/
class Component {

  /*** Protected data members ***/
  protected:
    char*     mType;    /*Type of the component */
    char*     mName;   /* Name of the component */
    Compound* mHolder; /* Holder component, if any */
    const LogicalClock* mClock;

  /*** Constructors & Dectructors ***/
  public:
    Component(const char*, const char*,
              Compound*,
              const LogicalClock* = NULL);
    virtual ~Component();


  /*** Public Data-Access Methods ***/
  public:
    inline char*     type() const   {return mType; }
    inline char*     name() const   { return mName; }
    inline Compound* holder() const { return mHolder; }
    inline const LogicalClock* clock()  const { return mClock; }
  /*** Public Utility Methods ***/
  public:
    virtual bool isAtom()     const  = 0;
    virtual bool isCompound() const  = 0;
    virtual void getBusyAtoms(Scheduler*)=0;

    virtual void setup(Scheduler*) = 0;
    virtual void createDefaultThreads() = 0;
    virtual void dump(FILE* file) const = 0;
};

#endif

