#ifndef __HIDDEN_H_
#define __HIDDEN_H_

class Compound;
class Atom;
class SThread;
class Port;
class Connector;

namespace Hidden {
    void markMaster(Compound* system, const char* port_path);
    void markSlave (Compound* system, const char* port_path);
    bool isMaster(Port* port);
    bool isSlave(Port* port);
    void markMaster(Port* master);
    void masterUpperMarkHidden(Port* master, Connector* conn);
    void masterLowerMarkHidden(Port* master, Connector* conn);
    void markSlave(Port* slave);
    void slaveUpperCollectRoots(Port* slave, Connector* conn);
    void riseToRootConnectors(Port* origin, Port* current, void (*callBack)(Port* origin, Connector* conn));
    void dropToBottomConnectors(Port* origin, Connector* conn,
                                        void (*cbEnter)(Port* origin, Connector* conn),
                                        void  (*cbExit)(Port* origin, Connector* conn));
    void checkProperMarking(Compound* system);
    bool hasNotifiedSlavePorts(Atom* atom);
    void notifySlavesToMasters(SThread* fromThread, Atom* atom);
    bool hasElectedMasterPort(Atom* atom);
    bool checkSlaveInteractionAtMaster(Port* master);
    void executeConnectorUp(Port* master);
    void executeConnectorDn(Port* master);
    void executeUp(Port* master, Connector* conn);
    void executeDn(Port* master, Connector* conn);
    void setSlavesExecutorThread(Port* master);
    void executeSlaves(Port* master);
    void notifyPortsAtSlaves(Port* master);
    bool mustNotifyFormerSlave(SThread* fromThread, Atom* atom);
    bool wasExecutedAsSlave(Atom* atom);
    void notifyFormerSlaveToOwnerThread(SThread* fromThread, Atom* atom );
    bool checkAndNotifyContainedFormerSlavesToEngine(SThread* holderThread);
    void markMasterType(Compound* system, const char* port_type_path);
    void markSlaveType(Compound* system, const char* port_type_path);
    void markType(Compound* system, const char* port_type_path,bool isMaster);
}

#endif
