

#ifndef PRIORITY_H
#define PRIORITY_H

#include "clock.h"


#include <vector>
#include <cstdio>

class Interaction;
class Compound;
class Atom;
class Connector;

/*******************************
 * Class: Priority
 * Description: 
 *******************************/
class Priority {
  public:
    enum Status {BUSY, READY};

    typedef std::vector<Atom*> atoms_type;

  /*** Protected data members ***/
  protected:
    Compound*    mHolder; /* holder compound */
    Interaction* mLow;    /* the inhibited   */
    Interaction* mHigh;   /* the inhibitor   */
    Status mStatus;
    atoms_type mSupport; // the atoms required to compute the guard

    mutable bool mEvaluated;  /* true iff the priority rule has been already evaluated */
    mutable bool mEvaluatedGuard; /* value of the guard of the last evaluation */
    mutable bool mApplied;    /* true iff the priority rule has been applied */
    BIP::Time    mDelay;      /* delay for the application of priority in real-time constraints */


  /*** Constructors & Dectructors ***/
  public:
    Priority(Compound*, Interaction*, Interaction*,
            const BIP::Time& = BIP::Time::ZERO);
  virtual ~Priority();

  /*** Public Data-Access Methods ***/
  public:
    inline Compound*    holder() const { return mHolder; }
    inline Interaction* low()    const { return mLow; }
    inline Interaction* high()   const { return mHigh; }
    inline Status       status() const { return mStatus; }
    inline void setStatus(const Status status) { mStatus = status; }
    inline bool         evaluated()                        const { return mEvaluated; }
    inline void         setEvaluated(bool evaluated)       const { mEvaluated = evaluated; }
    inline bool         evaluatedGuard()                   const { return mEvaluatedGuard; }
    inline void         setEvaluatedGuard(bool guard)      const { mEvaluatedGuard = guard; }
    inline bool         applied()                          const { return mApplied; }
    inline void         setApplied(bool applied)           const { mApplied = applied; }
    inline BIP::Time    delay()                            const { return mDelay; }

  /*** Public Utility Methods ***/
  public:
    virtual void setup();
    virtual bool allAtomsReady() const;
    virtual void dump(FILE* file) const;
    void reset() const;
    virtual void setupPrioLevel();
    void apply(Interaction&, Interaction&);

  /*** Public Methods invoked from appln. ***/
  public:
    virtual bool guard() const { return true; }
    virtual const Intervals& constraint() const;
};

#endif
