#ifndef THREAD_H_
#define THREAD_H_

#include <list>
#include <set>
#include <string>
#include <cassert>
#include <pthread.h>
#include <stdio.h>
#include "util.h"

class Atom;
class Port;
class Scheduler;
class Component;
class Compound;

//#define DEBUG_THREAD
#ifdef DEBUG_THREAD
#define debugthr_printf(...) printf(__VA_ARGS__)
#else
#define debugthr_printf(...)
#endif

/***********************************
 * Class: SThread
 * Description:  bip schedule thread where multiple atoms are executed.
 ***********************************/
class SThread {

  public:
    typedef Util::Set<Atom*> atom_set_type;
    typedef std::list<Atom*> atoms_type;
    typedef std::list<Port*> ports_type;

  protected:

    char*                  mName;  
    Scheduler*             mScheduler; 
    atoms_type             mAtoms;
    atom_set_type          mAtomSet;
    atoms_type             mNotifiedAtoms;
    atoms_type             mWaitingEngineAtoms;
    atom_set_type          mWaitingFormerSlaveAtoms; // atoms that are busy with slave ports
    atom_set_type          mWaitingMasterAtoms;      // atoms busy with master
    ports_type             mEngineElectedPorts;
    atoms_type             mFormerSlavesNotifiedToOwner;

    pthread_t              mThread;
    pthread_cond_t         mResume;
    pthread_mutex_t        mMutex;
  
    bool                   mIsBipThread;
 
  /*** Constructoris & Dectructors ***/
  public:
    SThread(const char* name, Scheduler* scheduler, bool is_bip_thread=false);
    virtual ~SThread();
 
  /*** Public Data-Access Methods ***/
  public:
    const char* name() { return mName; }
 
    pthread_cond_t* resume();
    pthread_mutex_t* mutex();  

    
  /*** Public Utility Methods ***/
  public:
            void              dump();
            void              attach(Compound* fromComponent, const std::string& path);
            void              attachComponent(Component* component);
            void              attachCompound( Compound* compound);
            void              attachAtom(Atom *p); /*      */
            bool              isBipThread() { return mIsBipThread; }

  /*** Public Methods visible from appln. ***/
  public:
    void launch();
    void notify(Atom *atom);
   
  protected:
    void    run();
    Port*   wait();

  public:
    void checkWaitingEngineAtoms();
    Port* checkWaitingMasterAtoms();
  
    inline Util::Set<Atom*>& waitingFormerSlaveAtoms() { return mWaitingFormerSlaveAtoms; }      
    inline Util::Set<Atom*>& waitingMasterAtoms() { return mWaitingMasterAtoms; }      

    inline atoms_type& formerSlavesNotifiedToOwner() { return mFormerSlavesNotifiedToOwner; }
  protected:
    static void* Run(void*);
    static int nThread;
    static int countAffinity_CPUs;
    int mCPU;
};


#endif /* THREAD_H_*/
