

#ifndef PRIORITY_H
#define PRIORITY_H

#include "clock.h"
#include "constraint.h"

class Interaction;
class Executable;
class Compound;

/*******************************
 * Class: Priority
 * Description: 
 *******************************/
class Priority {
  /*** Protected data members ***/
  protected:
    Compound*    mHolder;     /* holder compound  */
    Interaction* mLow;        /* the inhibited    */
    Interaction* mHigh;       /* the inhibitor    */
    mutable bool mEvaluated;  /* true iff the priority rule has been already evaluated */
    mutable bool mEvaluatedGuard; /* value of the guard of the last evaluation */
    mutable bool mApplied;    /* true iff the priority rule has been applied */
    BIP::Time    mDelay;      /* delay for the application of priority in real-time constraints */

  /*** Constructors & Dectructors ***/
  public:
    Priority(Compound*, 
	     Interaction*, 
             Interaction*,
             const BIP::Time& = BIP::Time::ZERO);
    
  virtual ~Priority();

  /*** Public Data-Access Methods ***/
  public:
    inline Compound*    holder()                           const { return mHolder; }
    inline Interaction* low()                              const { return mLow; }
    inline Interaction* high()                             const { return mHigh; }
    inline bool         evaluated()                        const { return mEvaluated; }
    inline void         setEvaluated(bool evaluated)       const { mEvaluated = evaluated; }
    inline bool         evaluatedGuard()                   const { return mEvaluatedGuard; }
    inline void         setEvaluatedGuard(bool guard)      const { mEvaluatedGuard = guard; }
    inline bool         applied()                          const { return mApplied; }
    inline void         setApplied(bool applied)           const { mApplied = applied; }
    inline BIP::Time    delay()                            const { return mDelay; }

  /*** Public Utility Methods ***/
  public:
            void reset() const;
    virtual void setup();
    virtual void setupPrioLevel();
            void apply(Executable&, Executable&);

  /*** Public Methods invoked from appln. ***/
  public:
    virtual       bool       guard()      const;
    virtual const Intervals& constraint() const;
};

#endif
