#!/bin/bash

EXPECTED_ARGS=4
E_BADARGS=65
E_NOFILE=66

if [ $# -ne $EXPECTED_ARGS ]
then
  echo ""
  echo "-----------------------"
  echo " `basename $0`         "
  echo "-----------------------"
  echo ""  
  echo "Usage: `basename $0` input_jobs_filename time_in_engine num_processors output_filename "
  echo " time_in_engine = engine transition cost in MICROSECONDS"
  echo "example: `basename $0` fppn_tg.jobs 1000 2 SCHED_TOOL_OUT"
  echo ""
  exit $E_BADARGS
fi

if [[ ! -f "$1" ]]
then
    echo "File $1 does not exist!"
    exit $E_NOFILE
fi

INPUT_FILE=$1
TIME_IN_ENGINE=$2
NUM_PROC=$3
OUTPUT_FILE=$4


SCALED_FILE=${INPUT_FILE}-IN-us
TRANS_FILE=${SCALED_FILE}-w-TRANS

LOG=${OUTPUT_FILE}.LOG


runScaleInstance  $INPUT_FILE 1000 $SCALED_FILE
runInsertTransitionJobs $SCALED_FILE $TRANS_FILE

echo "Utilization info: " > ${LOG}
runDeltaLoad $TRANS_FILE $TIME_IN_ENGINE >> ${LOG}
echo "Mixed-mode utilization:" >> ${LOG}
runDeltaLoad -m $TRANS_FILE $TIME_IN_ENGINE >> ${LOG}
cat ${LOG}

runDeltaScheduler $TRANS_FILE $TIME_IN_ENGINE $NUM_PROC $OUTPUT_FILE 
